/*******************************************************************************

    HEASARC Database Cone (HDBCONE) utility.

    AUTHOR: Song Yom (HSTX)
    DATE:   06/94

    MODIFICATION HISTORY:
    DATE    PROGRAMMER      DESCRIPTION
    ----    ----------      -----------
    12/94   Song Yom        added alternate host connection capability
    06/95   Song Yom        removed host dependent port number definition

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "csioc.h"

#ifdef sun
#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1
#endif

/*
    TCP/IP communication socket host name and port number
*/

#define HOST "legacy.gsfc.nasa.gov"
#define PORT 56839

/*
    Alternate host environment variable name
*/

#define HDBHOST "HDBHOST"

/*
    Maximum length of data buffer
*/

#define MAXLEN CSIOC_SIZE

/*
    Argument parameters
*/

#define MINARGC 6
#define MAXARGC 7
#define NDATABASE 1
#define NRA 2
#define NDEC 3
#define NRADIUS 4
#define NCOLUMN 5

#ifdef sun
main(argc,argv)
    int argc;
    char *argv[];
#else
main(int argc,char *argv[])
#endif
{
    int status = EXIT_FAILURE;
    int count;
    int sd;
    int length;
    char *ph;
    char host[MAXLEN];
    char file[MAXLEN];
    char ibuffer[MAXLEN];
    char obuffer[MAXLEN];
    FILE *fp;

/*
    Verify command line arguments, establish data communication connection
    with the server, construct and send the parameter buffer to the server,
    and check the return status.  If successful, create and populate the
    output data file with data received from the server, and terminate the
    data communication connection with the server.  Write all messages
    (informational and error) to standard error.
*/

    if (argc >= MINARGC && argc <= MAXARGC)
    {
        if (ph = getenv(HDBHOST))
            strcpy(host,ph);
        else
            strcpy(host,HOST);
        fprintf(stderr,"connecting to %s ... ",host);
        if (csioc_connect(host,PORT,&sd) == CSIOC_SUCCESS)
        {
            fprintf(stderr,"success\n");
            obuffer[0] = NULL;
            for (count = 1;count < MINARGC;++count)
            {
                strcat(obuffer,argv[count]);
                strcat(obuffer," ");
            }
            obuffer[strlen(obuffer) - 1] = NULL;
            if (csioc_output(sd,obuffer,strlen(obuffer) + 1) ==
            CSIOC_SUCCESS && csioc_status(sd) == CSIOC_SUCCESS)
            {
                if (argc == MAXARGC)
                    strcpy(file,argv[count]);
                else
                    sprintf(file,"%s.data",argv[NDATABASE]);
                fprintf(stderr,"creating output file (%s) ... ",file);
                if (fp = fopen(file,"w"))
                {
                    count = 0;
                    while (csioc_input(sd,ibuffer,&length) == CSIOC_SUCCESS)
                    {
                        if (ibuffer[0] != '#')
                            ++count;
                        fprintf(fp,"%s\n",ibuffer);
                    }
                    fclose(fp);
                    fprintf(stderr,"%d records\n",count);
                    status = EXIT_SUCCESS;
                }
                else
                    fprintf(stderr,"unable to create output file (%s)\n",file);
            }
            else
                fprintf(stderr,"%s\n",csioc_getstatus());
            csioc_disconnect(sd);
        }
        else
            fprintf(stderr,"error\n");
    }
    else
        fprintf(stderr,"usage: %s database ra dec radius column[,...] [file]\n",argv[0]);

    exit(status);
}
